# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.azurestackhci import AzureStackHCIClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestAzureStackHCIClustersOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(AzureStackHCIClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_clusters_get(self, resource_group):
        response = self.client.clusters.get(
            resource_group_name=resource_group.name,
            cluster_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_clusters_create(self, resource_group):
        response = self.client.clusters.create(
            resource_group_name=resource_group.name,
            cluster_name="str",
            cluster={
                "location": "str",
                "id": "str",
                "identity": {
                    "type": "str",
                    "principalId": "str",
                    "tenantId": "str",
                    "userAssignedIdentities": {"str": {"clientId": "str", "principalId": "str"}},
                },
                "kind": "str",
                "name": "str",
                "properties": {
                    "aadApplicationObjectId": "str",
                    "aadClientId": "str",
                    "aadServicePrincipalObjectId": "str",
                    "aadTenantId": "str",
                    "billingModel": "str",
                    "billingProperties": {
                        "nextBillingModel": {
                            "billingModel": "str",
                            "capabilitiesEnabled": ["str"],
                            "trialDaysRemaining": 0.0,
                        }
                    },
                    "cloudId": "str",
                    "cloudManagementEndpoint": "str",
                    "clusterPattern": "str",
                    "confidentialVmProperties": {
                        "confidentialVmIntent": "str",
                        "confidentialVmStatus": "str",
                        "confidentialVmStatusSummary": "str",
                    },
                    "connectivityStatus": "str",
                    "desiredProperties": {"diagnosticLevel": "str", "windowsServerSubscription": "str"},
                    "identityProvider": "str",
                    "isManagementCluster": bool,
                    "isolatedVmAttestationConfiguration": {
                        "attestationResourceId": "str",
                        "attestationServiceEndpoint": "str",
                        "relyingPartyServiceEndpoint": "str",
                    },
                    "lastBillingTimestamp": "2020-02-20 00:00:00",
                    "lastSyncTimestamp": "2020-02-20 00:00:00",
                    "localAvailabilityZones": [{"localAvailabilityZoneName": "str", "nodes": ["str"]}],
                    "logCollectionProperties": {
                        "fromDate": "2020-02-20 00:00:00",
                        "lastLogGenerated": "2020-02-20 00:00:00",
                        "logCollectionSessionDetails": [
                            {
                                "correlationId": "str",
                                "endTimeCollected": "2020-02-20 00:00:00",
                                "logCollectionError": {"errorCode": "str", "errorMessage": "str"},
                                "logCollectionJobType": "str",
                                "logCollectionStatus": "str",
                                "logEndTime": "2020-02-20 00:00:00",
                                "logSize": 0,
                                "logStartTime": "2020-02-20 00:00:00",
                                "timeCollected": "2020-02-20 00:00:00",
                            }
                        ],
                        "toDate": "2020-02-20 00:00:00",
                    },
                    "provisioningState": "str",
                    "registrationTimestamp": "2020-02-20 00:00:00",
                    "remoteSupportProperties": {
                        "accessLevel": "str",
                        "expirationTimeStamp": "2020-02-20 00:00:00",
                        "remoteSupportNodeSettings": [
                            {
                                "arcResourceId": "str",
                                "connectionErrorMessage": "str",
                                "connectionStatus": "str",
                                "createdAt": "2020-02-20 00:00:00",
                                "state": "str",
                                "transcriptLocation": "str",
                                "updatedAt": "2020-02-20 00:00:00",
                            }
                        ],
                        "remoteSupportProvisioningState": "str",
                        "remoteSupportSessionDetails": [
                            {
                                "accessLevel": "str",
                                "duration": 0,
                                "nodeName": "str",
                                "sessionEndTime": "2020-02-20 00:00:00",
                                "sessionStartTime": "2020-02-20 00:00:00",
                                "transcriptLocation": "str",
                            }
                        ],
                        "remoteSupportType": "str",
                    },
                    "reportedProperties": {
                        "clusterId": "str",
                        "clusterName": "str",
                        "clusterType": "str",
                        "clusterVersion": "str",
                        "diagnosticLevel": "str",
                        "hardwareClass": "str",
                        "imdsAttestation": "str",
                        "lastUpdated": "2020-02-20 00:00:00",
                        "manufacturer": "str",
                        "msiExpirationTimeStamp": "2020-02-20 00:00:00",
                        "nodes": [
                            {
                                "coreCount": 0.0,
                                "ehcResourceId": "str",
                                "id": 0.0,
                                "lastLicensingTimestamp": "2020-02-20 00:00:00",
                                "manufacturer": "str",
                                "memoryInGiB": 0.0,
                                "model": "str",
                                "name": "str",
                                "nodeType": "str",
                                "oemActivation": "str",
                                "osDisplayVersion": "str",
                                "osName": "str",
                                "osVersion": "str",
                                "serialNumber": "str",
                                "windowsServerSubscription": "str",
                            }
                        ],
                        "oemActivation": "str",
                        "supportedCapabilities": ["str"],
                    },
                    "resourceProviderObjectId": "str",
                    "ring": "str",
                    "sdnProperties": {
                        "sdnApiAddress": "str",
                        "sdnDomainName": "str",
                        "sdnIntegrationIntent": "str",
                        "sdnStatus": "str",
                    },
                    "secretsLocations": [{"secretsLocation": "str", "secretsType": "str"}],
                    "serviceEndpoint": "str",
                    "softwareAssuranceProperties": {
                        "lastUpdated": "2020-02-20 00:00:00",
                        "softwareAssuranceIntent": "str",
                        "softwareAssuranceStatus": "str",
                    },
                    "status": "str",
                    "storageType": "str",
                    "trialDaysRemaining": 0.0,
                },
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "tags": {"str": "str"},
                "type": "str",
            },
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_clusters_update(self, resource_group):
        response = self.client.clusters.update(
            resource_group_name=resource_group.name,
            cluster_name="str",
            cluster={
                "identity": {
                    "type": "str",
                    "principalId": "str",
                    "tenantId": "str",
                    "userAssignedIdentities": {"str": {"clientId": "str", "principalId": "str"}},
                },
                "properties": {
                    "aadClientId": "str",
                    "aadTenantId": "str",
                    "cloudManagementEndpoint": "str",
                    "desiredProperties": {"diagnosticLevel": "str", "windowsServerSubscription": "str"},
                },
                "tags": {"str": "str"},
            },
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_clusters_begin_delete(self, resource_group):
        response = self.client.clusters.begin_delete(
            resource_group_name=resource_group.name,
            cluster_name="str",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_clusters_list_by_resource_group(self, resource_group):
        response = self.client.clusters.list_by_resource_group(
            resource_group_name=resource_group.name,
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_clusters_list_by_subscription(self, resource_group):
        response = self.client.clusters.list_by_subscription()
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_clusters_begin_update_secrets_locations(self, resource_group):
        response = self.client.clusters.begin_update_secrets_locations(
            resource_group_name=resource_group.name,
            cluster_name="str",
            body={"properties": [{"secretsLocation": "str", "secretsType": "str"}]},
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_clusters_begin_upload_certificate(self, resource_group):
        response = self.client.clusters.begin_upload_certificate(
            resource_group_name=resource_group.name,
            cluster_name="str",
            upload_certificate_request={"properties": {"certificates": ["str"]}},
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_clusters_begin_create_identity(self, resource_group):
        response = self.client.clusters.begin_create_identity(
            resource_group_name=resource_group.name,
            cluster_name="str",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_clusters_begin_extend_software_assurance_benefit(self, resource_group):
        response = self.client.clusters.begin_extend_software_assurance_benefit(
            resource_group_name=resource_group.name,
            cluster_name="str",
            software_assurance_change_request={"properties": {"softwareAssuranceIntent": "str"}},
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_clusters_begin_change_ring(self, resource_group):
        response = self.client.clusters.begin_change_ring(
            resource_group_name=resource_group.name,
            cluster_name="str",
            change_ring_request={"properties": {"targetRing": "str"}},
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_clusters_begin_trigger_log_collection(self, resource_group):
        response = self.client.clusters.begin_trigger_log_collection(
            resource_group_name=resource_group.name,
            cluster_name="str",
            log_collection_request={"properties": {"fromDate": "2020-02-20 00:00:00", "toDate": "2020-02-20 00:00:00"}},
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_clusters_begin_configure_remote_support(self, resource_group):
        response = self.client.clusters.begin_configure_remote_support(
            resource_group_name=resource_group.name,
            cluster_name="str",
            remote_support_request={
                "properties": {
                    "accessLevel": "str",
                    "expirationTimeStamp": "2020-02-20 00:00:00",
                    "remoteSupportType": "str",
                }
            },
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.billingbenefits import BillingBenefitsMgmtClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-billingbenefits
# USAGE
    python macc_validate_create_milestones_on_existing_macc.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = BillingBenefitsMgmtClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.benefit.validate(
        body={
            "benefits": [
                {
                    "benefitType": "MACC",
                    "properties": {
                        "billingAccountResourceId": "/providers/Microsoft.Billing/billingAccounts/{acctId:orgId}",
                        "commitment": {"amount": 20000, "currencyCode": "USD", "grain": "FullTerm"},
                        "endAt": "2026-07-31T23:59:59Z",
                        "entityType": "Primary",
                        "milestones": [
                            {"commitment": {"amount": 7000, "currencyCode": "USD"}, "endAt": "2024-10-31T23:59:59Z"},
                            {"commitment": {"amount": 13000, "currencyCode": "USD"}, "endAt": "2025-01-31T23:59:59Z"},
                        ],
                        "resourceId": "/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/myResourceGroup/providers/Microsoft.BillingBenefits/benefits/myBenefit",
                        "systemId": "13810867107109237",
                    },
                }
            ]
        },
    )
    print(response)


# x-ms-original-file: 2025-12-01-preview/MaccValidateCreateMilestonesOnExistingMacc.json
if __name__ == "__main__":
    main()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.dataprotection import DataProtectionMgmtClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestDataProtectionMgmtBackupVaultsOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(DataProtectionMgmtClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_backup_vaults_get(self, resource_group):
        response = self.client.backup_vaults.get(
            resource_group_name=resource_group.name,
            vault_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_backup_vaults_begin_create_or_update(self, resource_group):
        response = self.client.backup_vaults.begin_create_or_update(
            resource_group_name=resource_group.name,
            vault_name="str",
            parameters={
                "location": "str",
                "properties": {
                    "bcdrSecurityLevel": "str",
                    "featureSettings": {
                        "crossRegionRestoreSettings": {"state": "str"},
                        "crossSubscriptionRestoreSettings": {"state": "str"},
                    },
                    "isVaultProtectedByResourceGuard": bool,
                    "monitoringSettings": {"azureMonitorAlertSettings": {"alertsForAllJobFailures": "str"}},
                    "provisioningState": "str",
                    "replicatedRegions": ["str"],
                    "resourceGuardOperationRequests": ["str"],
                    "resourceMoveDetails": {
                        "completionTimeUtc": "str",
                        "operationId": "str",
                        "sourceResourcePath": "str",
                        "startTimeUtc": "str",
                        "targetResourcePath": "str",
                    },
                    "resourceMoveState": "str",
                    "secureScore": "str",
                    "securitySettings": {
                        "encryptionSettings": {
                            "infrastructureEncryption": "str",
                            "kekIdentity": {"identityId": "str", "identityType": "str"},
                            "keyVaultProperties": {"keyUri": "str"},
                            "state": "str",
                        },
                        "immutabilitySettings": {"state": "str"},
                        "softDeleteSettings": {"retentionDurationInDays": 0.0, "state": "str"},
                    },
                    "storageSettings": [{"datastoreType": "str", "type": "str"}],
                },
                "eTag": "str",
                "id": "str",
                "identity": {
                    "principalId": "str",
                    "tenantId": "str",
                    "type": "str",
                    "userAssignedIdentities": {"str": {"clientId": "str", "principalId": "str"}},
                },
                "name": "str",
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "tags": {"str": "str"},
                "type": "str",
            },
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_backup_vaults_begin_update(self, resource_group):
        response = self.client.backup_vaults.begin_update(
            resource_group_name=resource_group.name,
            vault_name="str",
            parameters={
                "identity": {
                    "principalId": "str",
                    "tenantId": "str",
                    "type": "str",
                    "userAssignedIdentities": {"str": {"clientId": "str", "principalId": "str"}},
                },
                "properties": {
                    "featureSettings": {
                        "crossRegionRestoreSettings": {"state": "str"},
                        "crossSubscriptionRestoreSettings": {"state": "str"},
                    },
                    "monitoringSettings": {"azureMonitorAlertSettings": {"alertsForAllJobFailures": "str"}},
                    "resourceGuardOperationRequests": ["str"],
                    "securitySettings": {
                        "encryptionSettings": {
                            "infrastructureEncryption": "str",
                            "kekIdentity": {"identityId": "str", "identityType": "str"},
                            "keyVaultProperties": {"keyUri": "str"},
                            "state": "str",
                        },
                        "immutabilitySettings": {"state": "str"},
                        "softDeleteSettings": {"retentionDurationInDays": 0.0, "state": "str"},
                    },
                },
                "tags": {"str": "str"},
            },
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_backup_vaults_begin_delete(self, resource_group):
        response = self.client.backup_vaults.begin_delete(
            resource_group_name=resource_group.name,
            vault_name="str",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_backup_vaults_get_in_subscription(self, resource_group):
        response = self.client.backup_vaults.get_in_subscription()
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_backup_vaults_get_in_resource_group(self, resource_group):
        response = self.client.backup_vaults.get_in_resource_group(
            resource_group_name=resource_group.name,
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_backup_vaults_check_name_availability(self, resource_group):
        response = self.client.backup_vaults.check_name_availability(
            resource_group_name=resource_group.name,
            location="str",
            parameters={"name": "str", "type": "str"},
        )

        # please add some check logic here by yourself
        # ...



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
"""Type stub for _patch.py (async client).

Overrides get_openai_client() return type so that evals.create() accepts
Azure-specific grader types in addition to the standard OpenAI graders.
"""

from typing import Any, Iterable, List, Union, Optional
from httpx import Timeout
from openai import NotGiven, Omit, AsyncOpenAI as AsyncOpenAIClient
from openai._types import Body, Query, Headers
from openai.resources.evals.evals import AsyncEvals
from openai.resources.evals.runs.runs import AsyncRuns
from openai.types.evals.create_eval_jsonl_run_data_source_param import CreateEvalJSONLRunDataSourceParam
from openai.types.evals.create_eval_completions_run_data_source_param import CreateEvalCompletionsRunDataSourceParam
from openai.types.evals.run_create_params import DataSourceCreateEvalResponsesRunDataSource
from openai.types.evals.run_create_response import RunCreateResponse
from openai.types.eval_create_params import (
    DataSourceConfigCustom,
    DataSourceConfigLogs,
    DataSourceConfigStoredCompletions,
    TestingCriterionLabelModel,
    TestingCriterionTextSimilarity,
    TestingCriterionPython,
    TestingCriterionScoreModel,
)
from openai.types.graders.string_check_grader_param import StringCheckGraderParam
from openai.types.eval_create_response import EvalCreateResponse
from openai.types.shared_params.metadata import Metadata
from ._client import AIProjectClient as AIProjectClientGenerated
from .operations import TelemetryOperations
from ..models import (
    AzureAIBenchmarkPreviewEvalRunDataSource,
    AzureAIDataSourceConfig,
    AzureAIResponsesEvalRunDataSource,
    EvalCsvRunDataSource,
    TestingCriterionAzureAIEvaluator,
    RedTeamEvalRunDataSource,
    TargetCompletionEvalRunDataSource,
    TracesPreviewEvalRunDataSource,
)

class _AzureAsyncEvalRuns(AsyncRuns):
    async def create(
        self,
        eval_id: str,
        *,
        data_source: Union[
            CreateEvalJSONLRunDataSourceParam,
            CreateEvalCompletionsRunDataSourceParam,
            DataSourceCreateEvalResponsesRunDataSource,
            AzureAIBenchmarkPreviewEvalRunDataSource,
            AzureAIResponsesEvalRunDataSource,
            EvalCsvRunDataSource,
            RedTeamEvalRunDataSource,
            TargetCompletionEvalRunDataSource,
            TracesPreviewEvalRunDataSource,
        ],
        metadata: Optional[Metadata] | Omit = ...,
        name: str | Omit = ...,
        extra_headers: Headers | None = ...,
        extra_query: Query | None = ...,
        extra_body: Body | None = ...,
        timeout: float | Timeout | None | NotGiven = ...,
    ) -> RunCreateResponse: ...

class _AzureAsyncEvals(AsyncEvals):
    async def create(
        self,
        *,
        data_source_config: Union[
            DataSourceConfigCustom, DataSourceConfigLogs, DataSourceConfigStoredCompletions, AzureAIDataSourceConfig
        ],
        testing_criteria: Iterable[
            Union[
                TestingCriterionLabelModel,
                StringCheckGraderParam,
                TestingCriterionTextSimilarity,
                TestingCriterionPython,
                TestingCriterionScoreModel,
                TestingCriterionAzureAIEvaluator,
            ]
        ],
        metadata: Optional[Metadata] | Omit | None = ...,
        name: str | Omit = ...,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | Timeout | NotGiven | None = ...,
    ) -> EvalCreateResponse: ...
    @property
    def runs(self) -> _AzureAsyncEvalRuns: ...

class AsyncOpenAI(AsyncOpenAIClient):
    @property
    def evals(self) -> _AzureAsyncEvals: ...

class AIProjectClient(AIProjectClientGenerated):
    telemetry: TelemetryOperations
    def get_openai_client(
        self, agent_name: Optional[str] = None, **kwargs: Any  # pylint: disable=unused-argument
    ) -> AsyncOpenAI: ...

# To make mypy happy... otherwise imports of the below result in mypy "attr-defined" error
__all__: List[str] = ["AIProjectClient"]

def patch_sdk() -> None: ...



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.containerregistry import ContainerRegistryManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-containerregistry
# USAGE
    python pipeline_run_create_export.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = ContainerRegistryManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.pipeline_runs.begin_create(
        resource_group_name="myResourceGroup",
        registry_name="myRegistry",
        pipeline_run_name="myPipelineRun",
        pipeline_run_create_parameters={
            "properties": {
                "request": {
                    "artifacts": [
                        "sourceRepository/hello-world",
                        "sourceRepository2@sha256:00000000000000000000000000000000000",
                    ],
                    "pipelineResourceId": "/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/myResourceGroup/providers/Microsoft.ContainerRegistry/registries/myRegistry/exportPipelines/myExportPipeline",
                    "target": {"name": "myblob.tar.gz", "type": "AzureStorageBlob"},
                }
            }
        },
    ).result()
    print(response)


# x-ms-original-file: 2026-01-01-preview/PipelineRunCreate_Export.json
if __name__ == "__main__":
    main()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.azurestackhci import AzureStackHCIClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestAzureStackHCIDeploymentSettingsOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(AzureStackHCIClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_deployment_settings_get(self, resource_group):
        response = self.client.deployment_settings.get(
            resource_group_name=resource_group.name,
            cluster_name="str",
            deployment_settings_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_deployment_settings_begin_create_or_update(self, resource_group):
        response = self.client.deployment_settings.begin_create_or_update(
            resource_group_name=resource_group.name,
            cluster_name="str",
            resource={
                "id": "str",
                "name": "str",
                "properties": {
                    "arcNodeResourceIds": ["str"],
                    "deploymentConfiguration": {
                        "scaleUnits": [
                            {
                                "deploymentData": {
                                    "adouPath": "str",
                                    "assemblyInfo": {
                                        "packageVersion": "str",
                                        "payload": [
                                            {"fileName": "str", "hash": "str", "identifier": "str", "url": "str"}
                                        ],
                                    },
                                    "cluster": {
                                        "azureServiceEndpoint": "str",
                                        "cloudAccountName": "str",
                                        "clusterPattern": "str",
                                        "hardwareClass": "str",
                                        "name": "str",
                                        "witnessPath": "str",
                                        "witnessType": "str",
                                    },
                                    "domainFqdn": "str",
                                    "hostNetwork": {
                                        "enableStorageAutoIp": bool,
                                        "intents": [
                                            {
                                                "adapter": ["str"],
                                                "adapterPropertyOverrides": {
                                                    "jumboPacket": "str",
                                                    "networkDirect": "str",
                                                    "networkDirectTechnology": "str",
                                                },
                                                "name": "str",
                                                "overrideAdapterProperty": bool,
                                                "overrideQosPolicy": bool,
                                                "overrideVirtualSwitchConfiguration": bool,
                                                "qosPolicyOverrides": {
                                                    "bandwidthPercentage_SMB": "str",
                                                    "priorityValue8021Action_Cluster": "str",
                                                    "priorityValue8021Action_SMB": "str",
                                                },
                                                "trafficType": ["str"],
                                                "virtualSwitchConfigurationOverrides": {
                                                    "enableIov": "str",
                                                    "loadBalancingAlgorithm": "str",
                                                },
                                            }
                                        ],
                                        "sanNetworks": {
                                            "clusterNetworkConfig": {
                                                "adapterIPConfig": [
                                                    {
                                                        "addressPrefix": "str",
                                                        "name": "str",
                                                        "networkAdapterName": "str",
                                                        "vlanId": 0,
                                                    }
                                                ],
                                                "adapterProperties": {
                                                    "bandwidthPercentageSmb": 0,
                                                    "jumboPacket": 0,
                                                    "priorityValue8021ActionCluster": 0,
                                                    "priorityValue8021ActionSmb": 0,
                                                },
                                            }
                                        },
                                        "storageConnectivitySwitchless": bool,
                                        "storageNetworks": [
                                            {
                                                "name": "str",
                                                "networkAdapterName": "str",
                                                "storageAdapterIPInfo": [
                                                    {"ipv4Address": "str", "physicalNode": "str", "subnetMask": "str"}
                                                ],
                                                "vlanId": "str",
                                            }
                                        ],
                                    },
                                    "identityProvider": "str",
                                    "infrastructureNetwork": [
                                        {
                                            "dnsServerConfig": "str",
                                            "dnsServers": ["str"],
                                            "dnsZones": [{"dnsForwarder": ["str"], "dnsZoneName": "str"}],
                                            "gateway": "str",
                                            "ipPools": [{"endingAddress": "str", "startingAddress": "str"}],
                                            "subnetMask": "str",
                                            "useDhcp": bool,
                                        }
                                    ],
                                    "isManagementCluster": bool,
                                    "localAvailabilityZones": [{"localAvailabilityZoneName": "str", "nodes": ["str"]}],
                                    "namingPrefix": "str",
                                    "observability": {
                                        "episodicDataUpload": bool,
                                        "euLocation": bool,
                                        "streamingDataClient": bool,
                                    },
                                    "optionalServices": {"customLocation": "str"},
                                    "physicalNodes": [{"ipv4Address": "str", "name": "str"}],
                                    "sdnIntegration": {
                                        "networkController": {
                                            "macAddressPoolStart": "str",
                                            "macAddressPoolStop": "str",
                                            "networkVirtualizationEnabled": bool,
                                        }
                                    },
                                    "secrets": [{"eceSecretName": "str", "secretLocation": "str", "secretName": "str"}],
                                    "secretsLocation": "str",
                                    "securitySettings": {
                                        "bitlockerBootVolume": bool,
                                        "bitlockerDataVolumes": bool,
                                        "credentialGuardEnforced": bool,
                                        "driftControlEnforced": bool,
                                        "drtmProtection": bool,
                                        "hvciProtection": bool,
                                        "sideChannelMitigationEnforced": bool,
                                        "smbClusterEncryption": bool,
                                        "smbSigningEnforced": bool,
                                        "wdacEnforced": bool,
                                    },
                                    "storage": {
                                        "configurationMode": "str",
                                        "s2d": {"overprovisioningRatio": "str", "volumeType": "str"},
                                        "san": {"infraPerfLunId": "str", "infraVolLunId": "str"},
                                        "storageType": "str",
                                    },
                                },
                                "sbePartnerInfo": {
                                    "credentialList": [
                                        {"eceSecretName": "str", "secretLocation": "str", "secretName": "str"}
                                    ],
                                    "partnerProperties": [{"name": "str", "value": "str"}],
                                    "sbeDeploymentInfo": {
                                        "family": "str",
                                        "publisher": "str",
                                        "sbeManifestCreationDate": "2020-02-20 00:00:00",
                                        "sbeManifestSource": "str",
                                        "version": "str",
                                    },
                                },
                            }
                        ],
                        "version": "str",
                    },
                    "deploymentMode": "str",
                    "operationType": "str",
                    "provisioningState": "str",
                    "reportedProperties": {
                        "deploymentStatus": {
                            "status": "str",
                            "steps": [
                                {
                                    "description": "str",
                                    "endTimeUtc": "str",
                                    "exception": ["str"],
                                    "fullStepIndex": "str",
                                    "name": "str",
                                    "startTimeUtc": "str",
                                    "status": "str",
                                    "steps": [...],
                                }
                            ],
                        },
                        "validationStatus": {
                            "status": "str",
                            "steps": [
                                {
                                    "description": "str",
                                    "endTimeUtc": "str",
                                    "exception": ["str"],
                                    "fullStepIndex": "str",
                                    "name": "str",
                                    "startTimeUtc": "str",
                                    "status": "str",
                                    "steps": [...],
                                }
                            ],
                        },
                    },
                },
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "type": "str",
            },
            deployment_settings_name="str",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_deployment_settings_begin_delete(self, resource_group):
        response = self.client.deployment_settings.begin_delete(
            resource_group_name=resource_group.name,
            cluster_name="str",
            deployment_settings_name="str",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_deployment_settings_list_by_clusters(self, resource_group):
        response = self.client.deployment_settings.list_by_clusters(
            resource_group_name=resource_group.name,
            cluster_name="str",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.deviceregistry import DeviceRegistryMgmtClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-deviceregistry
# USAGE
    python update_namespace_discovered_device.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = DeviceRegistryMgmtClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.namespace_discovered_devices.begin_update(
        resource_group_name="myResourceGroup",
        namespace_name="my-namespace-1",
        discovered_device_name="my-discovereddevice-1",
        properties={
            "properties": {
                "endpoints": {
                    "outbound": {
                        "assigned": {
                            "newIothubEndpoint": {
                                "address": "https://iothub-for-dps.azure-devices.net",
                                "endpointType": "Microsoft.Devices/IotHubs",
                            }
                        }
                    }
                }
            }
        },
    ).result()
    print(response)


# x-ms-original-file: 2026-03-01-preview/Update_NamespaceDiscoveredDevice.json
if __name__ == "__main__":
    main()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.artifactsigning import ArtifactSigningMgmtClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestArtifactSigningMgmtCertificateProfilesOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ArtifactSigningMgmtClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_certificate_profiles_get(self, resource_group):
        response = self.client.certificate_profiles.get(
            resource_group_name=resource_group.name,
            account_name="str",
            profile_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_certificate_profiles_begin_create(self, resource_group):
        response = self.client.certificate_profiles.begin_create(
            resource_group_name=resource_group.name,
            account_name="str",
            profile_name="str",
            resource={
                "id": "str",
                "name": "str",
                "properties": {
                    "identityValidationId": "str",
                    "profileType": "str",
                    "certificates": [
                        {
                            "createdDate": "str",
                            "enhancedKeyUsage": "str",
                            "expiryDate": "str",
                            "revocation": {
                                "effectiveAt": "2020-02-20 00:00:00",
                                "failureReason": "str",
                                "reason": "str",
                                "remarks": "str",
                                "requestedAt": "2020-02-20 00:00:00",
                                "status": "str",
                            },
                            "serialNumber": "str",
                            "status": "str",
                            "subjectName": "str",
                            "thumbprint": "str",
                        }
                    ],
                    "includeCity": bool,
                    "includeCountry": bool,
                    "includePostalCode": bool,
                    "includeState": bool,
                    "includeStreetAddress": bool,
                    "provisioningState": "str",
                    "status": "str",
                },
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "type": "str",
            },
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_certificate_profiles_begin_delete(self, resource_group):
        response = self.client.certificate_profiles.begin_delete(
            resource_group_name=resource_group.name,
            account_name="str",
            profile_name="str",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_certificate_profiles_list_by_code_signing_account(self, resource_group):
        response = self.client.certificate_profiles.list_by_code_signing_account(
            resource_group_name=resource_group.name,
            account_name="str",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_certificate_profiles_revoke_certificate(self, resource_group):
        response = self.client.certificate_profiles.revoke_certificate(
            resource_group_name=resource_group.name,
            account_name="str",
            profile_name="str",
            body={
                "effectiveAt": "2020-02-20 00:00:00",
                "reason": "str",
                "serialNumber": "str",
                "thumbprint": "str",
                "remarks": "str",
            },
        )

        # please add some check logic here by yourself
        # ...



# The MIT License (MIT)
# Copyright (c) Microsoft Corporation. All rights reserved.

import test_config
from azure.cosmos import CosmosClient as CosmosSyncClient

cosmos_sync_client = CosmosSyncClient(test_config.TestConfig.host, test_config.TestConfig.masterKey)


def pytest_configure(config):
    """
    Allows plugins and conftest files to perform initial configuration.
    This hook is called for every plugin and initial conftest
    file after command line options have been parsed.
    """


def pytest_sessionstart(session):
    """
    Called after the Session object has been created and
    before performing collection and entering the run test loop.
    """
    config = test_config.TestConfig
    config.get_account_info(cosmos_sync_client)
    config.create_database_if_not_exist(cosmos_sync_client)
    config.create_single_partition_container_if_not_exist(cosmos_sync_client)
    config.create_multi_partition_container_if_not_exist(cosmos_sync_client)
    config.create_single_partition_prefix_pk_container_if_not_exist(cosmos_sync_client)
    config.create_multi_partition_prefix_pk_container_if_not_exist(cosmos_sync_client)

def pytest_sessionfinish(session, exitstatus):
    """
    Called after whole test run finished, right before
    returning the exit status to the system.
    """
    config = test_config.TestConfig
    config.try_delete_database(cosmos_sync_client)


def pytest_unconfigure(config):
    """
    called before test process is exited.
    """


import pytest


@pytest.fixture(autouse=True)
def _reset_shared_pk_range_cache():
    """Reset module-level shared partition-key-range cache between tests.

    The shared cache (introduced for the cross-client memory optimisation)
    is process-global state. Without this fixture, state from one test
    (cached routing maps, per-(loop, collection) locks, refcounts) leaks
    into subsequent tests, causing order-dependent failures and flakiness
    in any test that asserts on cache contents or _ReadPartitionKeyRanges
    call counts.

    We clear after the test runs so the test under observation can still
    exercise the normal population behaviour.
    """
    yield
    # Local import to avoid pulling these modules in at conftest collection
    # time (some environments treat conftest import errors as fatal).
    from azure.cosmos._routing import routing_map_provider as _sync_pmp
    from azure.cosmos._routing.aio import routing_map_provider as _async_pmp

    # Clear the *contents* of each per-endpoint cache dict, not the registry
    # itself. Long-lived test fixtures (class-level CosmosClient) hold strong
    # references to the inner dicts via ``_collection_routing_map_by_item``;
    # if we ``.clear()`` the outer registry, a freshly-constructed client for
    # the same endpoint creates a brand-new inner dict and the dict-identity
    # invariant that test_shared_cache_integration relies on is broken.
    # Same reasoning for ``_shared_collection_locks``.
    for pmp in (_sync_pmp, _async_pmp):
        with pmp._shared_cache_lock:  # pylint: disable=protected-access
            for cache in pmp._shared_routing_map_cache.values():  # pylint: disable=protected-access
                cache.clear()
            for locks in pmp._shared_collection_locks.values():  # pylint: disable=protected-access
                locks.clear()



from datetime import datetime
from enum import Enum
import os


class ServiceVersion(str, Enum):

    V2019_02_02 = "2019-02-02"
    V2019_07_07 = "2019-07-07"
    V2019_10_10 = "2019-10-10"
    V2019_12_12 = "2019-12-12"
    V2020_02_10 = "2020-02-10"
    V2020_04_08 = "2020-04-08"
    V2020_06_12 = "2020-06-12"
    V2020_08_04 = "2020-08-04"


service_version_map = {
    "V2019_02_02": ServiceVersion.V2019_02_02,
    "V2019_07_07": ServiceVersion.V2019_07_07,
    "V2019_10_10": ServiceVersion.V2019_10_10,
    "V2019_12_12": ServiceVersion.V2019_12_12,
    "V2020_02_10": ServiceVersion.V2020_02_10,
    "V2020_04_08": ServiceVersion.V2020_04_08,
    "V2020_06_12": ServiceVersion.V2020_06_12,
    "V2020_08_04": ServiceVersion.V2020_08_04,
    "LATEST": ServiceVersion.V2020_08_04,
    "LATEST_PLUS_1": ServiceVersion.V2020_06_12,
}


def is_version_before(test_version):
    """Return True if the current version is after a given one or if the
    service version is not set.
    """
    current_version = service_version_map.get(os.environ.get("AZURE_LIVE_TEST_SERVICE_VERSION"))
    if not current_version:
        return True
    current_version_data = datetime.strptime(current_version, "%Y-%m-%d")
    test_version_minimum = datetime.strptime(test_version, "%Y-%m-%d")
    ret = current_version_data < test_version_minimum
    return ret



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential
from azure.mgmt.botservice import AzureBotService

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-botservice
# USAGE
    python list_bots_by_resource_group.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = AzureBotService(
        credential=DefaultAzureCredential(),
        subscription_id="subscription-id",
    )

    response = client.bots.list_by_resource_group(
        resource_group_name="OneResourceGroupName",
    )
    for item in response:
        print(item)


# x-ms-original-file: specification/botservice/resource-manager/Microsoft.BotService/stable/2022-09-15/examples/ListBotsByResourceGroup.json
if __name__ == "__main__":
    main()



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
"""Offline unit tests for ``azure.ai.projects.operations._patch_models``.

These tests do not contact the Foundry service. They cover the patch helpers
``_extract_pending_upload_targets`` and ``_run_azcopy``, and the orchestration
performed by ``create`` (mocking ``pending_upload``, ``pending_create_version``
and ``get`` on the base class).
"""

from __future__ import annotations

import os
import subprocess
from pathlib import Path
from types import SimpleNamespace
from unittest import mock

import pytest

from azure.core.exceptions import ResourceNotFoundError

from azure.ai.projects.operations._patch_models import BetaModelsOperations

# ---------------------------------------------------------------------------
# _extract_pending_upload_targets
# ---------------------------------------------------------------------------


class TestExtractPendingUploadTargets:
    def test_datastore_wire_shape(self):
        """The service currently returns ``blobReferenceForConsumption``."""
        payload = {
            "blobReferenceForConsumption": {
                "blobUri": "https://acct.blob.core.windows.net/c/path",
                "credential": {"sasUri": "https://acct.blob.core.windows.net/c/path?sig=abc"},
            },
            "temporaryDataReferenceId": "abc-123",
        }
        sas, blob, pid = BetaModelsOperations._extract_pending_upload_targets(payload)
        assert sas == payload["blobReferenceForConsumption"]["credential"]["sasUri"]
        assert blob == payload["blobReferenceForConsumption"]["blobUri"]
        assert pid == "abc-123"

    def test_modeled_wire_shape(self):
        """The SDK-modeled shape uses ``blobReference`` / ``pendingUploadId``."""
        payload = {
            "blobReference": {
                "blobUri": "https://acct.blob.core.windows.net/c/path",
                "credential": {"sasUri": "https://acct.blob.core.windows.net/c/path?sig=xyz"},
            },
            "pendingUploadId": "modeled-id",
        }
        sas, blob, pid = BetaModelsOperations._extract_pending_upload_targets(payload)
        assert sas == payload["blobReference"]["credential"]["sasUri"]
        assert blob == payload["blobReference"]["blobUri"]
        assert pid == "modeled-id"

    def test_response_object_with_as_dict(self):
        """Accept anything exposing ``.as_dict()`` (the modeled response type)."""
        payload = {
            "blobReference": {
                "blobUri": "https://x/y",
                "credential": {"sasUri": "https://x/y?sas"},
            },
            "pendingUploadId": "id1",
        }
        response = SimpleNamespace(as_dict=lambda: payload)
        sas, blob, pid = BetaModelsOperations._extract_pending_upload_targets(response)
        assert (sas, blob, pid) == ("https://x/y?sas", "https://x/y", "id1")

    def test_missing_sas_uri_raises(self):
        payload = {
            "blobReferenceForConsumption": {"blobUri": "https://x/y", "credential": {}},
        }
        with pytest.raises(ValueError, match="SAS URI / blob URI"):
            BetaModelsOperations._extract_pending_upload_targets(payload)

    def test_missing_blob_uri_raises(self):
        payload = {
            "blobReferenceForConsumption": {"credential": {"sasUri": "https://x?sas"}},
        }
        with pytest.raises(ValueError, match="SAS URI / blob URI"):
            BetaModelsOperations._extract_pending_upload_targets(payload)

    def test_missing_pending_upload_id_is_none(self):
        payload = {
            "blobReferenceForConsumption": {
                "blobUri": "https://x/y",
                "credential": {"sasUri": "https://x/y?sas"},
            },
        }
        sas, blob, pid = BetaModelsOperations._extract_pending_upload_targets(payload)
        assert pid is None
        assert (sas, blob) == ("https://x/y?sas", "https://x/y")


# ---------------------------------------------------------------------------
# _run_azcopy
# ---------------------------------------------------------------------------


class TestRunAzcopy:
    def test_missing_azcopy_raises_runtime_error(self, tmp_path):
        src = tmp_path / "weights.bin"
        src.write_bytes(b"x")
        with mock.patch("azure.ai.projects.operations._patch_models.shutil.which", return_value=None):
            with pytest.raises(RuntimeError, match="azcopy"):
                BetaModelsOperations._run_azcopy(src, "https://x?sas")

    def test_explicit_path_overrides_shutil_which(self, tmp_path):
        src = tmp_path / "weights.bin"
        src.write_bytes(b"x")
        completed = SimpleNamespace(returncode=0, stdout="", stderr="")
        with mock.patch(
            "azure.ai.projects.operations._patch_models.subprocess.run",
            return_value=completed,
        ) as run, mock.patch("azure.ai.projects.operations._patch_models.shutil.which", return_value=None):
            BetaModelsOperations._run_azcopy(src, "https://x?sas", azcopy_path="/opt/azcopy")
        assert run.call_args.args[0][0] == "/opt/azcopy"

    def test_directory_source_uses_glob_arg(self, tmp_path):
        (tmp_path / "weights.bin").write_bytes(b"x")
        completed = SimpleNamespace(returncode=0, stdout="", stderr="")
        with mock.patch(
            "azure.ai.projects.operations._patch_models.subprocess.run",
            return_value=completed,
        ) as run, mock.patch("azure.ai.projects.operations._patch_models.shutil.which", return_value="/usr/bin/azcopy"):
            BetaModelsOperations._run_azcopy(tmp_path, "https://x?sas")
        cmd = run.call_args.args[0]
        assert cmd[0] == "/usr/bin/azcopy"
        assert cmd[1] == "copy"
        assert cmd[2] == str(tmp_path / "*")
        assert cmd[3] == "https://x?sas"
        assert "--from-to" in cmd
        assert cmd[cmd.index("--from-to") + 1] == "LocalBlob"
        assert "--recursive" in cmd

    def test_file_source_uses_file_arg(self, tmp_path):
        src = tmp_path / "weights.bin"
        src.write_bytes(b"x")
        completed = SimpleNamespace(returncode=0, stdout="", stderr="")
        with mock.patch(
            "azure.ai.projects.operations._patch_models.subprocess.run",
            return_value=completed,
        ) as run, mock.patch("azure.ai.projects.operations._patch_models.shutil.which", return_value="/usr/bin/azcopy"):
            BetaModelsOperations._run_azcopy(src, "https://x?sas")
        assert run.call_args.args[0][2] == str(src)

    def test_missing_source_raises_value_error(self, tmp_path):
        ghost = tmp_path / "does-not-exist"
        with mock.patch("azure.ai.projects.operations._patch_models.shutil.which", return_value="/usr/bin/azcopy"):
            with pytest.raises(ValueError, match="does not exist"):
                BetaModelsOperations._run_azcopy(ghost, "https://x?sas")

    def test_nonzero_exit_raises_runtime_error(self, tmp_path):
        src = tmp_path / "weights.bin"
        src.write_bytes(b"x")
        completed = SimpleNamespace(returncode=1, stdout="oops-stdout", stderr="oops-stderr")
        with mock.patch(
            "azure.ai.projects.operations._patch_models.subprocess.run",
            return_value=completed,
        ), mock.patch("azure.ai.projects.operations._patch_models.shutil.which", return_value="/usr/bin/azcopy"):
            with pytest.raises(RuntimeError, match="exited with code 1"):
                BetaModelsOperations._run_azcopy(src, "https://x?sas")


# ---------------------------------------------------------------------------
# create orchestration
# ---------------------------------------------------------------------------


def _make_ops() -> BetaModelsOperations:
    """Build a ``BetaModelsOperations`` without going through the client wiring."""
    return BetaModelsOperations.__new__(BetaModelsOperations)


def _pending_payload() -> dict:
    return {
        "blobReferenceForConsumption": {
            "blobUri": "https://acct.blob.core.windows.net/c/path",
            "credential": {"sasUri": "https://acct.blob.core.windows.net/c/path?sig=abc"},
        },
        "temporaryDataReferenceId": "pending-id-1",
    }


class TestCreateVersionOrchestration:
    @pytest.fixture(autouse=True)
    def _stub_azcopy_on_path(self):
        """Pretend azcopy is installed so the up-front validator passes."""
        with mock.patch(
            "azure.ai.projects.operations._patch_models.shutil.which",
            return_value="/usr/bin/azcopy",
        ):
            yield

    def test_models_create_runs_three_steps_in_order_and_returns_get_result(self, tmp_path):
        (tmp_path / "weights.bin").write_bytes(b"x")
        ops = _make_ops()

        committed = SimpleNamespace(
            id="model/abc",
            name="my-model",
            version="1",
            blob_uri="https://acct.blob.core.windows.net/c/path",
        )
        calls: list[str] = []

        def fake_pending_upload(**kwargs):
            calls.append("pending_upload")
            assert kwargs["name"] == "my-model"
            assert kwargs["version"] == "1"
            return _pending_payload()

        def fake_run_azcopy(source, sas_uri, *, azcopy_path=None):  # noqa: ARG001
            calls.append("azcopy")
            assert Path(source) == tmp_path
            assert sas_uri.startswith("https://")
            assert azcopy_path == "/custom/azcopy"

        def fake_create_async(**kwargs):
            calls.append("pending_create_version")
            assert kwargs["name"] == "my-model"
            assert kwargs["version"] == "1"
            body = kwargs["model_version"]
            # The blob_uri from the pending response is plumbed into the commit body.
            assert body.blob_uri == "https://acct.blob.core.windows.net/c/path"
            assert body.weight_type == "FullWeight"
            assert body.description == "desc"
            assert body.tags == {"k": "v"}

        def fake_get(**kwargs):
            calls.append("get")
            assert kwargs["name"] == "my-model"
            assert kwargs["version"] == "1"
            return committed

        with mock.patch.object(ops, "pending_upload", side_effect=fake_pending_upload), mock.patch.object(
            BetaModelsOperations, "_run_azcopy", staticmethod(fake_run_azcopy)
        ), mock.patch.object(ops, "pending_create_version", side_effect=fake_create_async), mock.patch.object(
            ops, "get", side_effect=fake_get
        ):
            result = ops.create(
                name="my-model",
                version="1",
                source=tmp_path,
                weight_type="FullWeight",
                description="desc",
                tags={"k": "v"},
                azcopy_path="/custom/azcopy",
            )

        assert result is committed
        assert calls == ["pending_upload", "azcopy", "pending_create_version", "get"]

    def test_models_create_wait_for_commit_false_returns_none_and_does_not_poll(self, tmp_path):
        (tmp_path / "weights.bin").write_bytes(b"x")
        ops = _make_ops()
        get_mock = mock.Mock()

        with mock.patch.object(ops, "pending_upload", return_value=_pending_payload()), mock.patch.object(
            BetaModelsOperations, "_run_azcopy", staticmethod(lambda *a, **kw: None)
        ), mock.patch.object(ops, "pending_create_version", return_value=None), mock.patch.object(ops, "get", get_mock):
            result = ops.create(
                name="m",
                version="1",
                source=tmp_path,
                wait_for_commit=False,
            )

        assert result is None
        get_mock.assert_not_called()

    def test_models_create_polls_until_get_succeeds(self, tmp_path):
        (tmp_path / "weights.bin").write_bytes(b"x")
        ops = _make_ops()
        committed = SimpleNamespace(name="m", version="1")

        get_mock = mock.Mock(
            side_effect=[
                ResourceNotFoundError(message="not yet"),
                ResourceNotFoundError(message="still not yet"),
                committed,
            ]
        )

        with mock.patch.object(ops, "pending_upload", return_value=_pending_payload()), mock.patch.object(
            BetaModelsOperations, "_run_azcopy", staticmethod(lambda *a, **kw: None)
        ), mock.patch.object(ops, "pending_create_version", return_value=None), mock.patch.object(
            ops, "get", get_mock
        ), mock.patch(
            "azure.ai.projects.operations._patch_models.time.sleep"
        ) as sleep:
            result = ops.create(
                name="m",
                version="1",
                source=tmp_path,
                polling_interval=0.01,
            )

        assert result is committed
        assert get_mock.call_count == 3
        assert sleep.call_count == 2

    def test_models_create_polling_timeout_raises_runtime_error(self, tmp_path):
        (tmp_path / "weights.bin").write_bytes(b"x")
        ops = _make_ops()

        # First a real time.monotonic call (start), then an over-the-deadline one.
        times = iter([1000.0, 1000.0, 9999.0])
        with mock.patch.object(ops, "pending_upload", return_value=_pending_payload()), mock.patch.object(
            BetaModelsOperations, "_run_azcopy", staticmethod(lambda *a, **kw: None)
        ), mock.patch.object(ops, "pending_create_version", return_value=None), mock.patch.object(
            ops, "get", side_effect=ResourceNotFoundError(message="never")
        ), mock.patch(
            "azure.ai.projects.operations._patch_models.time.monotonic",
            side_effect=lambda: next(times),
        ), mock.patch(
            "azure.ai.projects.operations._patch_models.time.sleep"
        ):
            with pytest.raises(RuntimeError, match="did not appear within"):
                ops.create(
                    name="m",
                    version="1",
                    source=tmp_path,
                    polling_timeout=1.0,
                )

    def test_models_create_missing_source_raises_before_calling_service(self, tmp_path):
        ops = _make_ops()
        ghost = tmp_path / "does-not-exist"
        pending = mock.Mock()
        with mock.patch.object(ops, "pending_upload", pending):
            with pytest.raises(ValueError, match="does not exist"):
                ops.create(name="m", version="1", source=ghost)
        pending.assert_not_called()


# ---------------------------------------------------------------------------
# _validate_create_inputs
# ---------------------------------------------------------------------------


class TestValidateCreateVersionInputs:
    @pytest.fixture(autouse=True)
    def _stub_azcopy_on_path(self):
        with mock.patch(
            "azure.ai.projects.operations._patch_models.shutil.which",
            return_value="/usr/bin/azcopy",
        ):
            yield

    def _kwargs(self, **overrides):
        defaults = dict(
            name="m",
            version="1",
            source="/tmp/x",
            azcopy_path=None,
            wait_for_commit=True,
            polling_timeout=300.0,
            polling_interval=2.0,
        )
        defaults.update(overrides)
        return defaults

    def test_valid_directory_source_returns_path(self, tmp_path):
        (tmp_path / "weights.bin").write_bytes(b"x")
        result = BetaModelsOperations._validate_create_inputs(**self._kwargs(source=tmp_path))
        assert result == tmp_path

    @pytest.mark.parametrize("bad_name", ["", "   ", None, 123])
    def test_empty_or_non_string_name_raises(self, tmp_path, bad_name):
        (tmp_path / "weights.bin").write_bytes(b"x")
        with pytest.raises(ValueError, match="`name`"):
            BetaModelsOperations._validate_create_inputs(**self._kwargs(name=bad_name, source=tmp_path))

    @pytest.mark.parametrize("bad_version", ["", "   ", None, 1])
    def test_empty_or_non_string_version_raises(self, tmp_path, bad_version):
        (tmp_path / "weights.bin").write_bytes(b"x")
        with pytest.raises(ValueError, match="`version`"):
            BetaModelsOperations._validate_create_inputs(**self._kwargs(version=bad_version, source=tmp_path))

    def test_empty_directory_raises(self, tmp_path):
        with pytest.raises(ValueError, match="directory is empty"):
            BetaModelsOperations._validate_create_inputs(**self._kwargs(source=tmp_path))

    def test_empty_file_raises(self, tmp_path):
        empty = tmp_path / "weights.bin"
        empty.write_bytes(b"")
        with pytest.raises(ValueError, match="file is empty"):
            BetaModelsOperations._validate_create_inputs(**self._kwargs(source=empty))

    @pytest.mark.parametrize("bad_timeout", [0, -1.0])
    def test_non_positive_polling_timeout_raises(self, tmp_path, bad_timeout):
        (tmp_path / "weights.bin").write_bytes(b"x")
        with pytest.raises(ValueError, match="polling_timeout"):
            BetaModelsOperations._validate_create_inputs(**self._kwargs(source=tmp_path, polling_timeout=bad_timeout))

    @pytest.mark.parametrize("bad_interval", [0, -1.0])
    def test_non_positive_polling_interval_raises(self, tmp_path, bad_interval):
        (tmp_path / "weights.bin").write_bytes(b"x")
        with pytest.raises(ValueError, match="polling_interval"):
            BetaModelsOperations._validate_create_inputs(**self._kwargs(source=tmp_path, polling_interval=bad_interval))

    def test_polling_params_skipped_when_wait_for_commit_false(self, tmp_path):
        (tmp_path / "weights.bin").write_bytes(b"x")
        # Negative polling values are tolerated when not waiting for commit.
        result = BetaModelsOperations._validate_create_inputs(
            **self._kwargs(source=tmp_path, wait_for_commit=False, polling_timeout=-1, polling_interval=-1)
        )
        assert result == tmp_path

    def test_missing_azcopy_raises(self, tmp_path):
        (tmp_path / "weights.bin").write_bytes(b"x")
        with mock.patch(
            "azure.ai.projects.operations._patch_models.shutil.which",
            return_value=None,
        ):
            with pytest.raises(RuntimeError, match="azcopy"):
                BetaModelsOperations._validate_create_inputs(**self._kwargs(source=tmp_path))

    def test_models_create_validates_before_calling_pending_upload(self, tmp_path):
        """Validation runs before any service operation."""
        (tmp_path / "weights.bin").write_bytes(b"x")
        ops = _make_ops()
        pending = mock.Mock()
        with mock.patch.object(ops, "pending_upload", pending), mock.patch(
            "azure.ai.projects.operations._patch_models.shutil.which", return_value=None
        ):
            with pytest.raises(RuntimeError, match="azcopy"):
                ops.create(name="m", version="1", source=tmp_path)
        pending.assert_not_called()



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.deviceregistry import DeviceRegistryMgmtClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-deviceregistry
# USAGE
    python policies_activate_bring_your_own_root.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = DeviceRegistryMgmtClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    client.policies.begin_activate_bring_your_own_root(
        resource_group_name="rgdeviceregistry",
        namespace_name="mynamespace",
        policy_name="mypolicy",
        body={
            "certificateChain": "-----BEGIN CERTIFICATE-----\nMIIDXTCCAkWgAwIBAgIJAKL0UG+mRkmWMA0GCSqGSIb3DQEBCwUAMEUxCzAJBgNV\nBAYTAkFVMRMwEQYDVQQIDApTb21lLVN0YXRlMSEwHwYDVQQKDBhJbnRlcm5ldCBX\naWRnaXRzIFB0eSBMdGQwHhcNMjQwMTAxMDAwMDAwWhcNMjUwMTAxMDAwMDAwWjBF\nMQswCQYDVQQGEwJBVTETMBEGA1UECAwKU29tZS1TdGF0ZTEhMB8GA1UECgwYSW50\nZXJuZXQgV2lkZ2l0cyBQdHkgTHRkMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIB\nCgKCAQEAw...\n-----END CERTIFICATE-----\n-----BEGIN CERTIFICATE-----\nMIIDXTCCAkWgAwIBAgIJAKL0UG+mRkmXMA0GCSqGSIb3DQEBCwUAMEUxCzAJBgNV\nBAYTAkFVMRMwEQYDVQQIDApTb21lLVN0YXRlMSEwHwYDVQQKDBhJbnRlcm5ldCBX\naWRnaXRzIFB0eSBMdGQwHhcNMjQwMTAxMDAwMDAwWhcNMjUwMTAxMDAwMDAwWjBF\nMQswCQYDVQQGEwJBVTETMBEGA1UECAwKU29tZS1TdGF0ZTEhMB8GA1UECgwYSW50\nZXJuZXQgV2lkZ2l0cyBQdHkgTHRkMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIB\nCgKCAQEAx...\n-----END CERTIFICATE-----"
        },
    ).result()


# x-ms-original-file: 2026-03-01-preview/Policies_ActivateBringYourOwnRoot.json
if __name__ == "__main__":
    main()



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.
"""Contract tests for POST /responses endpoint behavior."""

from __future__ import annotations

from typing import Any

from starlette.testclient import TestClient

from azure.ai.agentserver.responses import ResponsesAgentServerHost
from tests._helpers import poll_until


def _noop_response_handler(request: Any, context: Any, cancellation_signal: Any):
    """Minimal handler used to wire the hosting surface in contract tests."""

    async def _events():
        if False:  # pragma: no cover - required to keep async-generator shape.
            yield None

    return _events()


def _build_client() -> TestClient:
    app = ResponsesAgentServerHost()
    app.response_handler(_noop_response_handler)
    return TestClient(app)


def test_create__returns_json_response_for_non_streaming_success() -> None:
    client = _build_client()

    response = client.post(
        "/responses",
        json={
            "model": "gpt-4o-mini",
            "input": "hello",
            "stream": False,
            "store": True,
            "background": False,
        },
    )

    assert response.status_code == 200
    payload = response.json()
    assert isinstance(payload.get("id"), str)
    assert payload["id"].startswith("caresp_")
    assert payload.get("response_id") == payload.get("id")
    # agent_reference may be empty/absent when the request doesn't include one
    agent_ref = payload.get("agent_reference")
    assert agent_ref is None or isinstance(agent_ref, dict)
    assert payload.get("object") == "response"
    assert payload.get("status") in {"completed", "in_progress", "queued"}
    assert "sequence_number" not in payload


def test_create__preserves_client_supplied_identity_fields() -> None:
    client = _build_client()

    response = client.post(
        "/responses",
        json={
            "model": "gpt-4o-mini",
            "input": "hello",
            "response_id": "caresp_1234567890abcdef00ABCDEFGHIJKLMNOPQRSTUVWXYZ012345",
            "agent_reference": {
                "type": "agent_reference",
                "name": "custom-agent",
                "version": "v1",
            },
            "stream": False,
            "store": True,
            "background": False,
        },
    )

    assert response.status_code == 200
    payload = response.json()
    assert payload.get("id") == "caresp_1234567890abcdef00ABCDEFGHIJKLMNOPQRSTUVWXYZ012345"
    assert payload.get("response_id") == "caresp_1234567890abcdef00ABCDEFGHIJKLMNOPQRSTUVWXYZ012345"
    assert payload.get("agent_reference") == {
        "type": "agent_reference",
        "name": "custom-agent",
        "version": "v1",
    }


def test_create__rejects_invalid_response_id_format() -> None:
    client = _build_client()

    response = client.post(
        "/responses",
        json={
            "model": "gpt-4o-mini",
            "input": "hello",
            "response_id": "bad-id",
            "stream": False,
            "store": True,
            "background": False,
        },
    )

    assert response.status_code == 400
    payload = response.json()
    assert payload["error"].get("type") == "invalid_request_error"
    assert payload["error"].get("param") == "response_id"


def test_create__rejects_invalid_agent_reference_shape() -> None:
    client = _build_client()

    response = client.post(
        "/responses",
        json={
            "model": "gpt-4o-mini",
            "input": "hello",
            "agent_reference": {"type": "not_agent_reference", "name": "bad"},
            "stream": False,
            "store": True,
            "background": False,
        },
    )

    assert response.status_code == 400
    payload = response.json()
    assert payload["error"].get("type") == "invalid_request_error"
    assert payload["error"].get("param") == "agent_reference.type"


def test_create__returns_structured_400_for_invalid_payload() -> None:
    client = _build_client()

    response = client.post(
        "/responses",
        json={
            "background": True,
            "store": False,
        },
    )

    assert response.status_code == 400
    payload = response.json()
    error = payload.get("error")
    assert isinstance(error, dict)
    assert error.get("type") == "invalid_request_error"


def test_create__store_false_response_is_not_visible_via_get() -> None:
    client = _build_client()

    create_response = client.post(
        "/responses",
        json={
            "model": "gpt-4o-mini",
            "input": "hello",
            "stream": False,
            "store": False,
            "background": False,
        },
    )

    assert create_response.status_code == 200
    response_id = create_response.json()["id"]

    get_response = client.get(f"/responses/{response_id}")
    assert get_response.status_code == 404


def test_create__background_mode_returns_immediate_then_reaches_terminal_state() -> None:
    client = _build_client()

    create_response = client.post(
        "/responses",
        json={
            "model": "gpt-4o-mini",
            "input": "hello",
            "stream": False,
            "store": True,
            "background": True,
        },
    )

    assert create_response.status_code == 200
    created_payload = create_response.json()
    # Phase 3: handler runs immediately, so POST may return completed when the
    # noop handler finishes quickly in the TestClient synchronous context.
    assert created_payload.get("status") in {"queued", "in_progress", "completed"}
    response_id = created_payload["id"]

    latest_snapshot: dict[str, Any] = {}

    def _is_terminal() -> bool:
        nonlocal latest_snapshot
        snapshot_response = client.get(f"/responses/{response_id}")
        if snapshot_response.status_code != 200:
            return False
        latest_snapshot = snapshot_response.json()
        return latest_snapshot.get("status") in {"completed", "failed", "incomplete", "cancelled"}

    ok, failure = poll_until(
        _is_terminal,
        timeout_s=5.0,
        interval_s=0.05,
        context_provider=lambda: {"last_status": latest_snapshot.get("status")},
        label="background create terminal transition",
    )
    assert ok, failure


def test_create__non_stream_returns_completed_response_with_output_items() -> None:
    from azure.ai.agentserver.responses.streaming._event_stream import ResponseEventStream

    def _output_producing_handler(request: Any, context: Any, cancellation_signal: Any):
        async def _events():
            stream = ResponseEventStream(response_id=context.response_id, model=getattr(request, "model", None))
            yield stream.emit_created()
            yield stream.emit_in_progress()

            message_item = stream.add_output_item_message()
            yield message_item.emit_added()

            text_content = message_item.add_text_content()
            yield text_content.emit_added()
            yield text_content.emit_delta("hello")
            yield text_content.emit_text_done()
            yield text_content.emit_done()
            yield message_item.emit_done()

            yield stream.emit_completed()

        return _events()

    app = ResponsesAgentServerHost()
    app.response_handler(_output_producing_handler)
    client = TestClient(app)

    response = client.post(
        "/responses",
        json={
            "model": "gpt-4o-mini",
            "input": "hello",
            "stream": False,
            "store": True,
            "background": False,
        },
    )

    assert response.status_code == 200
    payload = response.json()
    assert payload.get("status") == "completed"
    assert "sequence_number" not in payload
    assert isinstance(payload.get("output"), list)
    assert len(payload["output"]) == 1
    assert payload["output"][0].get("type") == "message"
    assert payload["output"][0].get("content", [])[0].get("type") == "output_text"
    assert payload["output"][0].get("content", [])[0].get("text") == "hello"


def test_create__background_non_stream_get_eventually_returns_output_items() -> None:
    from azure.ai.agentserver.responses.streaming._event_stream import ResponseEventStream

    def _output_producing_handler(request: Any, context: Any, cancellation_signal: Any):
        async def _events():
            stream = ResponseEventStream(response_id=context.response_id, model=getattr(request, "model", None))
            yield stream.emit_created()
            yield stream.emit_in_progress()

            message_item = stream.add_output_item_message()
            yield message_item.emit_added()

            text_content = message_item.add_text_content()
            yield text_content.emit_added()
            yield text_content.emit_delta("hello")
            yield text_content.emit_text_done()
            yield text_content.emit_done()
            yield message_item.emit_done()

            yield stream.emit_completed()

        return _events()

    app = ResponsesAgentServerHost()
    app.response_handler(_output_producing_handler)
    client = TestClient(app)

    create_response = client.post(
        "/responses",
        json={
            "model": "gpt-4o-mini",
            "input": "hello",
            "stream": False,
            "store": True,
            "background": True,
        },
    )

    assert create_response.status_code == 200
    response_id = create_response.json()["id"]

    latest_snapshot: dict[str, Any] = {}

    def _is_completed_with_output() -> bool:
        nonlocal latest_snapshot
        snapshot_response = client.get(f"/responses/{response_id}")
        if snapshot_response.status_code != 200:
            return False
        latest_snapshot = snapshot_response.json()
        output = latest_snapshot.get("output")
        return latest_snapshot.get("status") == "completed" and isinstance(output, list) and len(output) == 1

    ok, failure = poll_until(
        _is_completed_with_output,
        timeout_s=5.0,
        interval_s=0.05,
        context_provider=lambda: {
            "last_status": latest_snapshot.get("status"),
            "last_output_count": len(latest_snapshot.get("output", []))
            if isinstance(latest_snapshot.get("output"), list)
            else None,
        },
        label="background non-stream output availability",
    )
    assert ok, failure

    assert latest_snapshot["output"][0].get("type") == "message"
    assert latest_snapshot["output"][0].get("content", [])[0].get("type") == "output_text"
    assert latest_snapshot["output"][0].get("content", [])[0].get("text") == "hello"
    assert "sequence_number" not in latest_snapshot


def test_create__model_is_optional_and_resolved_to_empty_or_default() -> None:
    """B22 — model can be omitted. Resolution: request.model → default_model → empty string."""
    client = _build_client()

    response = client.post(
        "/responses",
        json={
            "input": "hello",
            "stream": False,
            "store": True,
            "background": False,
        },
    )

    assert response.status_code == 200
    payload = response.json()
    # B22: model should be present (possibly empty string or server default)
    assert "model" in payload
    assert isinstance(payload["model"], str)


def test_create__metadata_rejects_more_than_16_keys() -> None:
    """Metadata constraints: max 16 key-value pairs."""
    client = _build_client()

    metadata = {f"key_{i}": f"value_{i}" for i in range(17)}
    response = client.post(
        "/responses",
        json={
            "model": "gpt-4o-mini",
            "input": "hello",
            "metadata": metadata,
            "stream": False,
            "store": True,
            "background": False,
        },
    )

    assert response.status_code == 400
    payload = response.json()
    assert payload["error"]["type"] == "invalid_request_error"


def test_create__metadata_rejects_key_longer_than_64_chars() -> None:
    """Metadata constraints: key max 64 characters."""
    client = _build_client()

    metadata = {"a" * 65: "value"}
    response = client.post(
        "/responses",
        json={
            "model": "gpt-4o-mini",
            "input": "hello",
            "metadata": metadata,
            "stream": False,
            "store": True,
            "background": False,
        },
    )

    assert response.status_code == 400
    payload = response.json()
    assert payload["error"]["type"] == "invalid_request_error"


def test_create__metadata_rejects_value_longer_than_512_chars() -> None:
    """Metadata constraints: value max 512 characters."""
    client = _build_client()

    metadata = {"key": "v" * 513}
    response = client.post(
        "/responses",
        json={
            "model": "gpt-4o-mini",
            "input": "hello",
            "metadata": metadata,
            "stream": False,
            "store": True,
            "background": False,
        },
    )

    assert response.status_code == 400
    payload = response.json()
    assert payload["error"]["type"] == "invalid_request_error"


def test_create__validation_error_includes_details_array() -> None:
    """B29 — Invalid request returns 400 with details[] array."""
    client = _build_client()

    response = client.post(
        "/responses",
        json={
            "model": "gpt-4o-mini",
            "input": "hello",
            "stream": "not-a-bool",
            "store": True,
            "background": False,
        },
    )

    assert response.status_code == 400
    payload = response.json()
    error = payload.get("error")
    assert error is not None
    assert error.get("type") == "invalid_request_error"
    # B29: should have details[] array
    details = error.get("details")
    assert isinstance(details, list), f"Expected details[] array per B29, got: {type(details)}"
    assert len(details) >= 1
    for detail in details:
        assert detail.get("type") == "invalid_request_error"
        assert detail.get("code") == "invalid_value"
        assert "param" in detail
        assert "message" in detail


# ══════════════════════════════════════════════════════════
# B1, B2, B3: Request body edge cases
# ══════════════════════════════════════════════════════════


def test_create__returns_400_for_empty_body() -> None:
    """B1 — Empty request body → HTTP 400, error.type: invalid_request_error."""
    client = _build_client()

    response = client.post(
        "/responses",
        content=b"",
        headers={"Content-Type": "application/json"},
    )

    assert response.status_code == 400
    payload = response.json()
    assert isinstance(payload.get("error"), dict)
    assert payload["error"].get("type") == "invalid_request_error"
    assert payload["error"].get("code") == "invalid_request_error"


def test_create__returns_400_for_invalid_json_body() -> None:
    """B2 — Malformed JSON body → HTTP 400, error.type: invalid_request_error."""
    client = _build_client()

    response = client.post(
        "/responses",
        content=b"{invalid json",
        headers={"Content-Type": "application/json"},
    )

    assert response.status_code == 400
    payload = response.json()
    assert isinstance(payload.get("error"), dict)
    assert payload["error"].get("type") == "invalid_request_error"
    assert payload["error"].get("code") == "invalid_request_error"


def test_create__ignores_unknown_fields_in_request_body() -> None:
    """B3 — Unknown fields are ignored for forward compatibility → HTTP 200."""
    client = _build_client()

    response = client.post(
        "/responses",
        json={
            "model": "gpt-4o-mini",
            "input": "hello",
            "stream": False,
            "store": True,
            "background": False,
            "foo": "bar",
            "unknown_nested": {"key": "value"},
        },
    )

    assert response.status_code == 200
    payload = response.json()
    assert payload.get("object") == "response"


# ══════════════════════════════════════════════════════════
# Task 4.1 — _process_handler_events sync contract tests
# ══════════════════════════════════════════════════════════


def test_sync_handler_exception_returns_500() -> None:
    """T5 — Handler raises an exception; stream=False → HTTP 500.

    B8 / B13 for sync mode: any handler exception surfaces as HTTP 500.
    """

    def _raising_handler(request: Any, context: Any, cancellation_signal: Any):
        async def _events():
            raise RuntimeError("Simulated handler failure")
            if False:  # pragma: no cover
                yield None

        return _events()

    _app = ResponsesAgentServerHost()
    _app.response_handler(_raising_handler)
    client = TestClient(_app, raise_server_exceptions=False)

    response = client.post(
        "/responses",
        json={"model": "gpt-4o-mini", "input": "hello", "stream": False, "store": True, "background": False},
    )

    assert response.status_code == 500
    # Server errors must return a structured error envelope
    payload = response.json()
    assert isinstance(payload.get("error"), dict), "500 must include a JSON error envelope"
    assert payload["error"].get("type") == "server_error"
    assert payload["error"].get("code") == "server_error"


def test_sync_no_terminal_event_still_completes() -> None:
    """T6 — Handler yields response.created + response.in_progress but no terminal.

    stream=False → HTTP 200, status=failed.

    S-015: When the handler completes without emitting a terminal event, the library
    synthesises a ``response.failed`` terminal.  Sync callers receive HTTP 200 with
    a "failed" response body (not HTTP 500).
    """
    from azure.ai.agentserver.responses.streaming._event_stream import ResponseEventStream

    def _no_terminal_handler(request: Any, context: Any, cancellation_signal: Any):
        async def _events():
            stream = ResponseEventStream(response_id=context.response_id, model=getattr(request, "model", None))
            yield stream.emit_created()
            yield stream.emit_in_progress()
            # Intentionally omit terminal event (response.completed / response.failed)

        return _events()

    _app = ResponsesAgentServerHost()
    _app.response_handler(_no_terminal_handler)
    client = TestClient(_app)

    response = client.post(
        "/responses",
        json={"model": "gpt-4o-mini", "input": "hello", "stream": False, "store": True, "background": False},
    )

    assert response.status_code == 200, (
        f"S-015: sync no-terminal handler must return HTTP 200, got {response.status_code}"
    )
    payload = response.json()
    assert payload.get("status") == "failed", (
        f"S-015: synthesised terminal must set status to 'failed', got {payload.get('status')!r}"
    )


# ══════════════════════════════════════════════════════════
# Phase 5 — Task 5.1: FR-006 / FR-007 first-event contract first-event contract tests
# ══════════════════════════════════════════════════════════


def test_s007_wrong_first_event_sync() -> None:
    """T1 — Handler yields response.in_progress as first event; stream=False → HTTP 500.

    FR-006: The first event MUST be response.created.  Violations are treated as
    pre-creation errors (B8) and map to HTTP 500 in sync mode.
    Uses a raw dict to bypass ResponseEventStream internal ordering validation so
    the orchestrator's _check_first_event_contract is the authority under test.
    """

    def _wrong_first_event_handler(request: Any, context: Any, cancellation_signal: Any):
        async def _events():
            # Raw dict bypasses ResponseEventStream validation so _check_first_event_contract runs
            yield {
                "type": "response.in_progress",
                "response": {
                    "status": "in_progress",
                    "object": "response",
                },
            }

        return _events()

    _app = ResponsesAgentServerHost()
    _app.response_handler(_wrong_first_event_handler)
    client = TestClient(_app, raise_server_exceptions=False)

    response = client.post(
        "/responses",
        json={"model": "gpt-4o-mini", "input": "hello", "stream": False, "store": True, "background": False},
    )

    assert response.status_code == 500, (
        f"FR-006 violation in sync mode must return HTTP 500, got {response.status_code}"
    )


def test_s007_wrong_first_event_stream() -> None:
    """T2 — Handler yields response.in_progress as first event; stream=True → SSE contains only 'error'.

    FR-006: Violation → single standalone error event; no response.created in stream.
    Uses a raw dict to bypass ResponseEventStream internal ordering validation.
    """

    def _wrong_first_event_handler(request: Any, context: Any, cancellation_signal: Any):
        async def _events():
            yield {
                "type": "response.in_progress",
                "response": {
                    "status": "in_progress",
                    "object": "response",
                },
            }

        return _events()

    _app = ResponsesAgentServerHost()
    _app.response_handler(_wrong_first_event_handler)
    client = TestClient(_app, raise_server_exceptions=False)

    import json as _json

    events: list[dict[str, Any]] = []
    with client.stream(
        "POST",
        "/responses",
        json={"model": "gpt-4o-mini", "input": "hello", "stream": True, "store": True, "background": False},
    ) as response:
        assert response.status_code == 200
        current_type: str | None = None
        current_data: str | None = None
        for line in response.iter_lines():
            if not line:
                if current_type is not None:
                    events.append({"type": current_type, "data": _json.loads(current_data) if current_data else {}})
                current_type = None
                current_data = None
                continue
            if line.startswith("event:"):
                current_type = line.split(":", 1)[1].strip()
            elif line.startswith("data:"):
                current_data = line.split(":", 1)[1].strip()
        if current_type is not None:
            events.append({"type": current_type, "data": _json.loads(current_data) if current_data else {}})

    event_types = [e["type"] for e in events]
    assert event_types == ["error"], (
        f"FR-006 violation in stream mode must produce exactly ['error'], got: {event_types}"
    )
    assert "response.created" not in event_types


def test_s008_mismatched_id_stream() -> None:
    """T3 — Handler yields response.created with wrong id; stream=True → SSE contains only 'error'.

    FR-006b: The id in response.created MUST equal the library-assigned response_id.
    """

    def _mismatched_id_handler(request: Any, context: Any, cancellation_signal: Any):
        async def _events():
            # Emit response.created with a deliberately wrong id
            yield {
                "type": "response.created",
                "response": {
                    "id": "caresp_WRONG00000000000000000000000000000000000000000000",
                    "response_id": "caresp_WRONG00000000000000000000000000000000000000000000",
                    "status": "queued",
                    "object": "response",
                },
            }

        return _events()

    _app = ResponsesAgentServerHost()
    _app.response_handler(_mismatched_id_handler)
    client = TestClient(_app, raise_server_exceptions=False)

    import json as _json

    events: list[dict[str, Any]] = []
    with client.stream(
        "POST",
        "/responses",
        json={"model": "gpt-4o-mini", "input": "hello", "stream": True, "store": True, "background": False},
    ) as response:
        assert response.status_code == 200
        current_type: str | None = None
        current_data: str | None = None
        for line in response.iter_lines():
            if not line:
                if current_type is not None:
                    events.append({"type": current_type, "data": _json.loads(current_data) if current_data else {}})
                current_type = None
                current_data = None
                continue
            if line.startswith("event:"):
                current_type = line.split(":", 1)[1].strip()
            elif line.startswith("data:"):
                current_data = line.split(":", 1)[1].strip()
        if current_type is not None:
            events.append({"type": current_type, "data": _json.loads(current_data) if current_data else {}})

    event_types = [e["type"] for e in events]
    assert event_types == ["error"], f"FR-006b violation must produce exactly ['error'], got: {event_types}"


def test_s009_terminal_status_on_created_stream() -> None:
    """T4 — Handler yields response.created with terminal status; stream=True → SSE contains only 'error'.

    FR-007: The status in response.created MUST be non-terminal (queued or in_progress).
    """

    def _terminal_on_created_handler(request: Any, context: Any, cancellation_signal: Any):
        async def _events():
            yield {
                "type": "response.created",
                "response": {
                    "status": "completed",
                    "object": "response",
                },
            }

        return _events()

    _app = ResponsesAgentServerHost()
    _app.response_handler(_terminal_on_created_handler)
    client = TestClient(_app, raise_server_exceptions=False)

    import json as _json

    events: list[dict[str, Any]] = []
    with client.stream(
        "POST",
        "/responses",
        json={"model": "gpt-4o-mini", "input": "hello", "stream": True, "store": True, "background": False},
    ) as response:
        assert response.status_code == 200
        current_type: str | None = None
        current_data: str | None = None
        for line in response.iter_lines():
            if not line:
                if current_type is not None:
                    events.append({"type": current_type, "data": _json.loads(current_data) if current_data else {}})
                current_type = None
                current_data = None
                continue
            if line.startswith("event:"):
                current_type = line.split(":", 1)[1].strip()
            elif line.startswith("data:"):
                current_data = line.split(":", 1)[1].strip()
        if current_type is not None:
            events.append({"type": current_type, "data": _json.loads(current_data) if current_data else {}})

    event_types = [e["type"] for e in events]
    assert event_types == ["error"], f"FR-007 violation must produce exactly ['error'], got: {event_types}"


def test_s007_valid_handler_not_affected() -> None:
    """T5 — Compliant handler emits response.created with correct id; stream=True → normal SSE flow.

    Regression: the FR-006/FR-007 validation must not block valid handlers.
    """
    from azure.ai.agentserver.responses.streaming._event_stream import ResponseEventStream

    def _compliant_handler(request: Any, context: Any, cancellation_signal: Any):
        async def _events():
            stream = ResponseEventStream(response_id=context.response_id, model=getattr(request, "model", None))
            yield stream.emit_created()
            yield stream.emit_completed()

        return _events()

    _app = ResponsesAgentServerHost()
    _app.response_handler(_compliant_handler)
    client = TestClient(_app)

    import json as _json

    events: list[dict[str, Any]] = []
    with client.stream(
        "POST",
        "/responses",
        json={"model": "gpt-4o-mini", "input": "hello", "stream": True, "store": True, "background": False},
    ) as response:
        assert response.status_code == 200
        current_type: str | None = None
        current_data: str | None = None
        for line in response.iter_lines():
            if not line:
                if current_type is not None:
                    events.append({"type": current_type, "data": _json.loads(current_data) if current_data else {}})
                current_type = None
                current_data = None
                continue
            if line.startswith("event:"):
                current_type = line.split(":", 1)[1].strip()
            elif line.startswith("data:"):
                current_data = line.split(":", 1)[1].strip()
        if current_type is not None:
            events.append({"type": current_type, "data": _json.loads(current_data) if current_data else {}})

    event_types = [e["type"] for e in events]
    assert "response.created" in event_types, (
        f"Compliant handler must not be blocked; expected response.created in: {event_types}"
    )
    assert "error" not in event_types, f"Compliant handler must not produce error event; got: {event_types}"



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------
"""
FILE: sample_update_analyzer.py

DESCRIPTION:
    This sample demonstrates how to update an existing custom analyzer, including updating
    its description and tags.

    The update_analyzer method allows you to modify certain properties of an existing analyzer.
    The following properties can be updated:
    - Description: Update the analyzer's description
    - Tags: Add or update tags

USAGE:
    python sample_update_analyzer.py

    Set the environment variables with your own values before running the sample:
    1) CONTENTUNDERSTANDING_ENDPOINT - the endpoint to your Content Understanding resource.
    2) CONTENTUNDERSTANDING_KEY - your Content Understanding API key (optional if using DefaultAzureCredential).
"""

import os
import time

from dotenv import load_dotenv
from azure.ai.contentunderstanding import ContentUnderstandingClient
from azure.ai.contentunderstanding.models import (
    ContentAnalyzer,
    ContentAnalyzerConfig,
    ContentFieldSchema,
    ContentFieldDefinition,
    ContentFieldType,
    GenerationMethod,
)
from azure.core.credentials import AzureKeyCredential
from azure.identity import DefaultAzureCredential

load_dotenv()


def main() -> None:
    endpoint = os.environ["CONTENTUNDERSTANDING_ENDPOINT"]
    key = os.getenv("CONTENTUNDERSTANDING_KEY")
    credential = AzureKeyCredential(key) if key else DefaultAzureCredential()

    client = ContentUnderstandingClient(endpoint=endpoint, credential=credential)

    # Create initial analyzer
    analyzer_id = f"my_analyzer_for_update_{int(time.time())}"

    print(f"Creating initial analyzer '{analyzer_id}'...")

    analyzer = ContentAnalyzer(
        base_analyzer_id="prebuilt-document",
        description="Initial description",
        config=ContentAnalyzerConfig(return_details=True),
        field_schema=ContentFieldSchema(
            name="demo_schema",
            description="Schema for update demo",
            fields={
                "company_name": ContentFieldDefinition(
                    type=ContentFieldType.STRING,
                    method=GenerationMethod.EXTRACT,
                    description="Name of the company",
                ),
            },
        ),
        models={"completion": "gpt-4.1"},
        tags={"tag1": "tag1_initial_value", "tag2": "tag2_initial_value"},
    )

    poller = client.begin_create_analyzer(
        analyzer_id=analyzer_id,
        resource=analyzer,
    )
    poller.result()
    print(f"Analyzer '{analyzer_id}' created successfully!")

    # [START update_analyzer]
    # First, get the current analyzer to preserve base analyzer ID
    current_analyzer = client.get_analyzer(analyzer_id=analyzer_id)

    # Display current analyzer information
    print("\nCurrent analyzer information:")
    print(f"  Description: {current_analyzer.description}")
    if current_analyzer.tags:
        tags_str = ", ".join(f"{k}={v}" for k, v in current_analyzer.tags.items())
        print(f"  Tags: {tags_str}")

    # Create an updated analyzer with new description and tags
    updated_analyzer = ContentAnalyzer(
        base_analyzer_id=current_analyzer.base_analyzer_id,
        description="Updated description",
        tags={
            "tag1": "tag1_updated_value",  # Update existing tag
            "tag3": "tag3_value",  # Add new tag
        },
    )

    # Update the analyzer
    print(f"\nUpdating analyzer '{analyzer_id}'...")
    client.update_analyzer(analyzer_id=analyzer_id, resource=updated_analyzer)

    # Verify the update
    updated = client.get_analyzer(analyzer_id=analyzer_id)
    print("\nUpdated analyzer information:")
    print(f"  Description: {updated.description}")
    if updated.tags:
        tags_str = ", ".join(f"{k}={v}" for k, v in updated.tags.items())
        print(f"  Tags: {tags_str}")
    # [END update_analyzer]

    # Clean up - delete the analyzer
    print(f"\nCleaning up: deleting analyzer '{analyzer_id}'...")
    client.delete_analyzer(analyzer_id=analyzer_id)
    print(f"Analyzer '{analyzer_id}' deleted successfully.")


if __name__ == "__main__":
    main()



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

import json
import os
from typing import Any, IO, Mapping, Optional, Union

from .._utils.model_base import Model, SdkJSONEncoder

# file-like tuple could be `(filename, IO (or bytes))` or `(filename, IO (or bytes), content_type)`
FileContent = Union[str, bytes, IO[str], IO[bytes]]

FileType = Union[
    # file (or bytes)
    FileContent,
    # (filename, file (or bytes))
    tuple[Optional[str], FileContent],
    # (filename, file (or bytes), content_type)
    tuple[Optional[str], FileContent, Optional[str]],
]


def serialize_multipart_data_entry(data_entry: Any) -> Any:
    if isinstance(data_entry, (list, tuple, dict, Model)):
        return json.dumps(data_entry, cls=SdkJSONEncoder, exclude_readonly=True)
    return data_entry


def _normalize_multipart_file_entry(field_name: str, entry: Any, index: int) -> Any:
    """Ensure each multipart file entry carries a filename so that it is encoded
    as a file part (with ``filename=``) rather than a plain form field.

    Servers commonly distinguish multipart file parts from form-data parts by
    the presence of ``filename=`` in the part's ``Content-Disposition`` header.
    When callers pass bare bytes / str / IO objects (e.g.
    ``Path("x.zip").read_bytes()``), the underlying HTTP client emits the part
    without a filename, which several Foundry endpoints reject with errors like
    "At least one file must be uploaded". This helper synthesizes a filename
    from the IO object's ``name`` attribute when available, otherwise falls
    back to a stable default.

    :param field_name: The multipart form field name, used as a fallback filename.
    :type field_name: str
    :param entry: The file entry to normalize. May be a tuple, bytes, str, or IO object.
    :type entry: any
    :param index: Position of the entry within its field's list, used to disambiguate fallback filenames.
    :type index: int
    :return: A ``(filename, entry)`` tuple if ``entry`` was not already a tuple, otherwise ``entry`` unchanged.
    :rtype: any
    """
    if isinstance(entry, tuple):
        return entry
    filename: Optional[str] = None
    name_attr = getattr(entry, "name", None)
    if isinstance(name_attr, str) and name_attr:
        filename = os.path.basename(name_attr)
    if not filename:
        filename = f"{field_name}_{index}" if index else field_name
    return (filename, entry)


def prepare_multipart_form_data(
    body: Mapping[str, Any], multipart_fields: list[str], data_fields: list[str]
) -> list[FileType]:
    files: list[FileType] = []

    # Append data fields first so they appear before file parts in the encoded
    # multipart body. Some streaming server-side parsers (e.g. the Foundry
    # hosted-agents `create_agent_version_from_code` endpoint) require small
    # JSON metadata parts to precede large binary file parts; otherwise they
    # report the metadata part as missing.
    for data_field in data_fields:
        data_entry = body.get(data_field)
        if data_entry:
            files.append((data_field, str(serialize_multipart_data_entry(data_entry))))

    for multipart_field in multipart_fields:
        multipart_entry = body.get(multipart_field)
        if isinstance(multipart_entry, list):
            for idx, e in enumerate(multipart_entry):
                files.append((multipart_field, _normalize_multipart_file_entry(multipart_field, e, idx)))
        elif multipart_entry:
            files.append((multipart_field, _normalize_multipart_file_entry(multipart_field, multipart_entry, 0)))

    return files



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_extract_key_phrases_async.py

DESCRIPTION:
    This sample demonstrates how to run **key phrase extraction** over text (async).

USAGE:
    python sample_extract_key_phrases_async.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_TEXT_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_TEXT_ENDPOINT
      - AZURE_TEXT_KEY
"""

# [START extract_key_phrases_async]
import os
import asyncio

from azure.identity.aio import DefaultAzureCredential
from azure.ai.textanalytics.aio import TextAnalysisClient
from azure.ai.textanalytics.models import (
    MultiLanguageTextInput,
    MultiLanguageInput,
    TextKeyPhraseExtractionInput,
    KeyPhraseActionContent,
    AnalyzeTextKeyPhraseResult,
)


async def sample_extract_key_phrases_async():
    # get settings
    endpoint = os.environ["AZURE_TEXT_ENDPOINT"]
    credential = DefaultAzureCredential()

    async with TextAnalysisClient(endpoint, credential=credential) as client:
        # Build input
        text_a = (
            "We love this trail and make the trip every year. The views are breathtaking and well worth the hike! "
            "Yesterday was foggy though, so we missed the spectacular views. We tried again today and it was "
            "amazing. Everyone in my family liked the trail although it was too challenging for the less "
            "athletic among us. Not necessarily recommended for small children. A hotel close to the trail "
            "offers services for childcare in case you want that."
        )

        body = TextKeyPhraseExtractionInput(
            text_input=MultiLanguageTextInput(
                multi_language_inputs=[MultiLanguageInput(id="A", text=text_a, language="en")]
            ),
            action_content=KeyPhraseActionContent(model_version="latest"),
        )

        result = await client.analyze_text(body=body)

        # Validate and print results
        if not isinstance(result, AnalyzeTextKeyPhraseResult):
            print("Unexpected result type.")
            return

        if result.results is None:
            print("No results returned.")
            return

        if result.results.documents is None or len(result.results.documents) == 0:
            print("No documents in the response.")
            return

        for doc in result.results.documents:
            print(f"\nDocument ID: {doc.id}")
            if doc.key_phrases:
                print("Key Phrases:")
                for phrase in doc.key_phrases:
                    print(f"  - {phrase}")
            else:
                print("No key phrases found for this document.")


# [END extract_key_phrases_async]


async def main():
    await sample_extract_key_phrases_async()


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.compute.aio import ComputeManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestComputeManagementDedicatedHostGroupsOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ComputeManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_dedicated_host_groups_get(self, resource_group):
        response = await self.client.dedicated_host_groups.get(
            resource_group_name=resource_group.name,
            host_group_name="str",
            api_version="2025-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_dedicated_host_groups_create_or_update(self, resource_group):
        response = await self.client.dedicated_host_groups.create_or_update(
            resource_group_name=resource_group.name,
            host_group_name="str",
            parameters={
                "location": "str",
                "id": "str",
                "name": "str",
                "properties": {
                    "platformFaultDomainCount": 0,
                    "additionalCapabilities": {"ultraSSDEnabled": bool},
                    "hosts": [{"id": "str"}],
                    "instanceView": {
                        "hosts": [
                            {
                                "assetId": "str",
                                "availableCapacity": {"allocatableVMs": [{"count": 0.0, "vmSize": "str"}]},
                                "name": "str",
                                "statuses": [
                                    {
                                        "code": "str",
                                        "displayStatus": "str",
                                        "level": "str",
                                        "message": "str",
                                        "time": "2020-02-20 00:00:00",
                                    }
                                ],
                            }
                        ]
                    },
                    "supportAutomaticPlacement": bool,
                },
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "tags": {"str": "str"},
                "type": "str",
                "zones": ["str"],
            },
            api_version="2025-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_dedicated_host_groups_update(self, resource_group):
        response = await self.client.dedicated_host_groups.update(
            resource_group_name=resource_group.name,
            host_group_name="str",
            parameters={
                "properties": {
                    "platformFaultDomainCount": 0,
                    "additionalCapabilities": {"ultraSSDEnabled": bool},
                    "hosts": [{"id": "str"}],
                    "instanceView": {
                        "hosts": [
                            {
                                "assetId": "str",
                                "availableCapacity": {"allocatableVMs": [{"count": 0.0, "vmSize": "str"}]},
                                "name": "str",
                                "statuses": [
                                    {
                                        "code": "str",
                                        "displayStatus": "str",
                                        "level": "str",
                                        "message": "str",
                                        "time": "2020-02-20 00:00:00",
                                    }
                                ],
                            }
                        ]
                    },
                    "supportAutomaticPlacement": bool,
                },
                "tags": {"str": "str"},
                "zones": ["str"],
            },
            api_version="2025-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_dedicated_host_groups_delete(self, resource_group):
        response = await self.client.dedicated_host_groups.delete(
            resource_group_name=resource_group.name,
            host_group_name="str",
            api_version="2025-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_dedicated_host_groups_list_by_resource_group(self, resource_group):
        response = self.client.dedicated_host_groups.list_by_resource_group(
            resource_group_name=resource_group.name,
            api_version="2025-11-01",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_dedicated_host_groups_list_by_subscription(self, resource_group):
        response = self.client.dedicated_host_groups.list_by_subscription(
            api_version="2025-11-01",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...
